@@ -19,6 +19,9 @@
 #include <string.h>
 #include <math.h>
 
+#include <time.h>
+#include <sys/time.h>
+
 #include "gpxview.h"
 #include "custom_size_renderer.h"
 #include "custom_rating_renderer.h"
@@ -845,20 +848,72 @@ static void search_result_free(gpx_t *result) {
 }
 
 #define MAX_HITS 50
-gpx_t *search_do(gpx_t *gpx, char *phrase, int what, gboolean local) {
+
+static time_t localize_time(time_t in) {
+  time_t ret;
+  char *tz;
+  struct tm *tm = localtime(&in);
+  
+  tz = getenv("TZ");
+  setenv("TZ", "", 1);
+  tzset();
+  ret = mktime(tm);
+  if (tz)
+    setenv("TZ", tz, 1);
+  else
+    unsetenv("TZ");
+  tzset();
+  return ret;
+}
+
+static int days_ago(time_t in) {
+  int day_in  = localize_time(in) / (60*60*24);
+  int day_now = localize_time(time(NULL)) / (60*60*24);
+
+  return day_now - day_in;
+}
+
+gpx_t *search_do(appdata_t *appdata, gpx_t *gpx, char *phrase, 
+		 int what, gboolean local) {
   /* walk through all caches */
       
   int hits = 0;
   gpx_t *found = malloc(sizeof(gpx_t));
   memset(found, 0, sizeof(gpx_t));
   cache_t **cacheP = &(found->cache);
+
+  struct timezone tz;
   
+  if(what & SEARCH_FINDS) {
+    time_t loc_now = localize_time(time(NULL));
+    printf("now: %ld days since 1/1/1970, days hour is %ld\n", 
+	   loc_now/(60*60*24), loc_now%(60*60*24)/(60*60));
+  }
+
   while(gpx && hits < MAX_HITS) {
+
+    /* we need all notes ... */
+    if(what & SEARCH_FINDS) {
+      notes_load_all(appdata, gpx);
+      gpx->notes_loaded = TRUE;      
+    }
+
     cache_t *cache = gpx->cache;
+
     while(cache && hits < MAX_HITS) {
       gboolean hit = FALSE;
 
-      if(cache->id && (what & SEARCH_ID) && 
+      if(what & SEARCH_FINDS) {
+	if(cache->notes && cache->notes->found ) {
+	  int days = days_ago(cache->notes->ftime);
+
+	  if(cache->id)
+	    printf("find of %s is %d days ago\n", cache->id, days);
+
+	  if(days <= appdata->search_days)
+	    hit = 1;
+	}
+      } else if(cache->id && (what & SEARCH_ID) && 
 	 strcasestr(cache->id, phrase)) 
 	hit = 1;
       else if(cache->name && (what & SEARCH_NAME) && 
@@ -894,71 +949,147 @@ gpx_t *search_do(gpx_t *gpx, char *phrase, int what, gboolean local) {
   return found;
 }
 
+typedef struct {
+  appdata_t *appdata;
+  GtkWidget *entry, *spinner;
+  GtkWidget *in_id, *in_name, *in_desc, *in_owner, *in_finds;
+} search_context_t;
+
+static void callback_finds_toggled(GtkWidget *widget, gpointer data ) {
+  search_context_t *context = (search_context_t*)data;
+
+  gboolean in_finds = gtk_toggle_button_get_active(
+	  GTK_TOGGLE_BUTTON(context->in_finds));
+
+  gtk_widget_set_sensitive(context->entry,    !in_finds);
+  gtk_widget_set_sensitive(context->in_id,    !in_finds);
+  gtk_widget_set_sensitive(context->in_name,  !in_finds);
+  gtk_widget_set_sensitive(context->in_desc,  !in_finds);
+  gtk_widget_set_sensitive(context->in_owner, !in_finds);
+  gtk_widget_set_sensitive(context->spinner,   in_finds);
+}
+
 static void 
 cb_menu_search(GtkWidget *window, gpointer data) {
   appdata_t *appdata = (appdata_t *)data;
 
+  search_context_t context;
+  memset(&context, 0, sizeof(search_context_t));
+  context.appdata = appdata;
+
   GtkWidget *dialog = gtk_dialog_new_with_buttons(_("Enter search phrase"),
   	    GTK_WINDOW(appdata->window), GTK_DIALOG_MODAL,
             GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
             GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
             NULL);
 
-  GtkWidget *entry, *in_id, *in_name, *in_desc, *in_owner;
-
   gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox),
 			      gtk_label_new(_("Search in:")));
 
-  in_id = gtk_check_button_new_with_label(_("Waypoint IDs"));
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(in_id), 
+  GtkWidget *table = gtk_table_new(2, 2, TRUE);
+  gtk_table_set_col_spacing(GTK_TABLE(table), 0, 8);
+
+  context.in_id = gtk_check_button_new_with_label(_("Waypoint IDs"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(context.in_id), 
 			       appdata->search & SEARCH_ID);
-  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), in_id);
-  in_name = gtk_check_button_new_with_label(_("Names"));
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(in_name), 
+  gtk_table_attach_defaults(GTK_TABLE(table), context.in_id, 0, 1, 0, 1);
+
+  context.in_name = gtk_check_button_new_with_label(_("Names"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(context.in_name), 
 			       appdata->search & SEARCH_NAME);
-  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), in_name);
-  in_desc = gtk_check_button_new_with_label(_("Descriptions"));
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(in_desc), 
+  gtk_table_attach_defaults(GTK_TABLE(table), context.in_name, 1, 2, 0, 1);
+
+  context.in_desc = gtk_check_button_new_with_label(_("Descriptions"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(context.in_desc), 
 			       appdata->search & SEARCH_DESC);
-  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), in_desc);
-  in_owner = gtk_check_button_new_with_label(_("Owner"));
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(in_owner), 
+  gtk_table_attach_defaults(GTK_TABLE(table), context.in_desc, 0, 1, 1, 2);
+
+  context.in_owner = gtk_check_button_new_with_label(_("Owner"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(context.in_owner), 
 			       appdata->search & SEARCH_OWNER);
-  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), in_owner);
+  gtk_table_attach_defaults(GTK_TABLE(table), context.in_owner, 1, 2, 1, 2);
+
+  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), table);
+
+  /* -------------------------------------------------------------- */
 
   gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox),
 			      gtk_label_new(_("Search for:")));
   gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox),
-			      entry = gtk_entry_new());
+			      context.entry = gtk_entry_new());
   if(appdata->search_str)
-    gtk_entry_set_text(GTK_ENTRY(entry), appdata->search_str);
+    gtk_entry_set_text(GTK_ENTRY(context.entry), appdata->search_str);
+
+  /* -------------------------------------------------------------- */
+
+  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
+			      gtk_hseparator_new());
+
+  GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
+
+  context.in_finds = gtk_check_button_new_with_label(_("Search finds for"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(context.in_finds), 
+			       appdata->search & SEARCH_FINDS);
+  gtk_box_pack_start_defaults(GTK_BOX(hbox), context.in_finds);
+  g_signal_connect(G_OBJECT(context.in_finds), "toggled",
+		   G_CALLBACK(callback_finds_toggled), &context);
+
+#ifndef USE_MAEMO
+  GtkObject *adj = gtk_adjustment_new(appdata->search_days, 0, 99, 1, 10, 10);
+  context.spinner = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 0);
+#else
+  context.spinner = hildon_number_editor_new(0, 99);
+  hildon_number_editor_set_value(HILDON_NUMBER_EDITOR(context.spinner), 
+				 appdata->search_days);
+#endif
+  gtk_box_pack_start_defaults(GTK_BOX(hbox), context.spinner);
+
+  gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new(_("days")));
+
+  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox);
+
+  /* -------------------------------------------------------------- */
 
   gtk_widget_show_all(dialog);
+  callback_finds_toggled(NULL, &context);
 
   if(GTK_RESPONSE_ACCEPT == gtk_dialog_run(GTK_DIALOG(dialog))) {
-    char *p = strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
+    char *p = strdup(gtk_entry_get_text(GTK_ENTRY(context.entry)));
 
     /* update saved search string */
     if(appdata->search_str) free(appdata->search_str);
     if(strlen(p) > 0)
       appdata->search_str = strdup(p);
 
-    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(in_id)))
+#ifndef USE_MAEMO
+    appdata->search_days = gtk_spin_button_get_value_as_int(
+		    GTK_SPIN_BUTTON(context.spinner));
+#else
+    appdata->search_days = hildon_number_editor_get_value(
+		    HILDON_NUMBER_EDITOR(context.spinner));
+#endif
+
+    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context.in_finds)))
+      appdata->search |=  SEARCH_FINDS;
+    else
+      appdata->search &= ~SEARCH_FINDS;
+
+    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context.in_id)))
       appdata->search |=  SEARCH_ID;
     else
       appdata->search &= ~SEARCH_ID;
 
-    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(in_name)))
+    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context.in_name)))
       appdata->search |=  SEARCH_NAME;
     else
       appdata->search &= ~SEARCH_NAME;
 
-    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(in_desc)))
+    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context.in_desc)))
       appdata->search |=  SEARCH_DESC;
     else
       appdata->search &= ~SEARCH_DESC;
  
-    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(in_owner)))
+    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context.in_owner)))
       appdata->search |=  SEARCH_OWNER;
     else
       appdata->search &= ~SEARCH_OWNER;
@@ -966,22 +1097,23 @@ cb_menu_search(GtkWidget *window, gpointer data) {
     gtk_widget_destroy(dialog);
 
     /* don't search if we are asked to search for nothing */
-    if(appdata->search && strlen(p) > 0) {
+    if(((appdata->search & (SEARCH_ID|SEARCH_NAME|SEARCH_DESC|SEARCH_OWNER)) &&
+	strlen(p) > 0) || (appdata->search & SEARCH_FINDS)) {
       gpx_t *found = NULL;
 
       printf("Search for %s (flags = %x)...\n", p, appdata->search);
 
 #ifndef USE_MAEMO
-      found = search_do(appdata->gpx, p, appdata->search, FALSE);
+      found = search_do(appdata, appdata->gpx, p, appdata->search, FALSE);
 
       /* do search result dialog here ... */
       cachelist_dialog(appdata, found);
       search_result_free(found);
 #else
       if(appdata->cur_gpx)
-	found = search_do(appdata->cur_gpx, p, appdata->search, TRUE);
+	found = search_do(appdata, appdata->cur_gpx, p, appdata->search, TRUE);
       else
-	found = search_do(appdata->gpx, p, appdata->search, FALSE);
+	found = search_do(appdata, appdata->gpx, p, appdata->search, FALSE);
 
       gtk_container_remove(GTK_CONTAINER(appdata->vbox), appdata->cur_view);
       appdata->cur_view = cachelist_create(appdata, found, NULL);
